###libraries 
library(stringr)
library(igraph)
library(betalink) 

###networks directory route
route<-"/home/Uniqueness/networks"

#stablishing the working directory
setwd(route)

#list of files- This function search for the csv files within the working directory
Files<-list.files(pattern="csv")

############################################################
nets<-list()
for(i in 1:length(Files)){
  nets[[i]]<-read.csv(Files[i],row.names = 1)  # tab delimited matrix
}
Names<-character()
for(i in 1:length(Files)){
  Names[i]<-str_sub(Files[i],end=-5)  
}
names(nets)<-Names
nets2Igraph<-list()
nets2Igraph<- lapply(nets,function(x){
  x<-graph.incidence(x)
})
names(nets2Igraph)<-Names
Meta<-list()
qM<-list()
N<-nets2Igraph
for(i in 1:length(N)){
  Meta[[i]] <- metaweb(N)
}
for(i in 1:length(N)){
  qM[[i]] <- metaweb(N[-i])
}

uniq<-function(N,M){

  Shar<-numeric()
  Uniq<-numeric()
  Tot<-numeric()
  M<-numeric()
  qMe<-numeric()
  for(i in 1:length(N)){
    n1<-N[[i]]
    n2<-qM[[i]]
    n3<-Meta[[i]]
    
    a<-unique(plyr::aaply(igraph::get.edgelist(n1), 1, function(x) stringr::str_c(x,collapse = "--", paste = "_")))
    b<-unique(plyr::aaply(igraph::get.edgelist(n2), 1, function(x) stringr::str_c(x,collapse = "--", paste = "_")))
    c<-unique(plyr::aaply(igraph::get.edgelist(n3), 1, function(x) stringr::str_c(x,collapse = "--", paste = "_")))
    Tot[[i]]<-length(a)
    Shar[[i]]<-length(which(b %in% a))
    Uniq[[i]]<-length(a)-Shar[[i]]
    M[[i]]<-length(c)
    qMe[[i]]<-M[[i]]-Uniq[[i]]+Shar[[i]]
    
  }
  
  data.frame(LocalNetwork=Names,Metaweb=M,QuasiM=qMe,Local=Tot,Shared=Shar,Unique=Uniq,LNU=Uniq/Tot, PShared=Shar/M)
}

uni = uniq(N,M)
make_readable_edgelist = function(x) {
  X = igraph::get.edgelist(x)
  return(paste(X[,1], X[,2]))
}
Shared_frequency<-function(x,y,z){ 
  e1 = make_readable_edgelist(x)
  e2 = make_readable_edgelist(y)
  
  i<-e2[which(e2 %in% e1)]
  
  l<-numeric()
  m<-numeric()
  for(k in 1:length(i)){
    for(j in 1:length(z)){
      l[j]<-any(make_readable_edgelist(z[[j]])==i[k])
    }
    m[k]<-sum(l)
  }
  return(m)
}
x1<-N
y1<-qM

SIF<-function(x1,y1){
  Com<-numeric()
for(i in 1:length(x1)){
  Com[i]<-(mean(Shared_frequency(x1[[i]],y1[[i]],x1))/(length(x1))) 
  }
data.frame(SIF=Com)
  }

sif = SIF(x1,y1)


uniqueness = data.frame(uni, sif)

#### Uniqueness- This function calculates nine descriptors:
# The first column is the identify of each network
# The second value returns the number of interactions within the metaweb (Metaweb). 
# The thrid value returns the number of interactions of the Quasimetaweb (QuasiM). 
# The fourth value resturns the number of interactions within the local network studied (Local).
# The fifth value returns the number of interaction shared between the local network and the metaweb (Shared).
# The sixth  value returns the number of unique interactions (not shared with the metaweb) of a local network (Unique). 
# The seventh value returns the proportion of unique interactions of a local network (LNU).
# The eighth value returns the proportion of interactions shared between a local network and the metaweb (PShared).
# The nineth value returns the how common are the shared interactions (SIF)
# NA means that a given network does not share interactions with the metaweb


uniqueness

